@@ -16,6 +16,7 @@
  */
 package org.apache.activemq.store.kahadb.disk.journal;
 
+import java.io.EOFException;
 import java.io.File;
 import java.io.FileNotFoundException;
 import java.io.FilenameFilter;
@@ -25,7 +26,6 @@ import java.io.UnsupportedEncodingException;
 import java.nio.ByteBuffer;
 import java.nio.channels.ClosedByInterruptException;
 import java.nio.channels.FileChannel;
-import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.Iterator;
@@ -89,12 +89,21 @@ public class Journal {
         // with corruption on recovery we have no faith in the content - slip to the next batch record or eof
         DataFileAccessor reader = accessorPool.openDataFileAccessor(dataFile);
         try {
-            int nextOffset = findNextBatchRecord(reader, recoveryPosition.getOffset() + 1);
-            Sequence sequence = new Sequence(recoveryPosition.getOffset(), nextOffset >= 0 ? nextOffset - 1 : dataFile.getLength() - 1);
-            LOG.warn("Corrupt journal records found in '" + dataFile.getFile() + "' between offsets: " + sequence);
+            RandomAccessFile randomAccessFile = reader.getRaf().getRaf();
+            randomAccessFile.seek(recoveryPosition.getOffset() + 1);
+            byte[] data = new byte[getWriteBatchSize()];
+            ByteSequence bs = new ByteSequence(data, 0, randomAccessFile.read(data));
+            int nextOffset = 0;
+            if (findNextBatchRecord(bs, randomAccessFile) >= 0) {
+                nextOffset = Math.toIntExact(randomAccessFile.getFilePointer() - bs.remaining());
+            } else {
+                nextOffset = Math.toIntExact(randomAccessFile.length());
+            }
+            Sequence sequence = new Sequence(recoveryPosition.getOffset(), nextOffset - 1);
+            LOG.warn("Corrupt journal records found in '{}' between offsets: {}", dataFile.getFile(), sequence);
 
             // skip corruption on getNextLocation
-            recoveryPosition.setOffset((int) sequence.getLast() + 1);
+            recoveryPosition.setOffset(nextOffset);
             recoveryPosition.setSize(-1);
 
             dataFile.corruptedBlocks.add(sequence);
@@ -108,6 +117,13 @@ public class Journal {
         return accessorPool;
     }
 
+    public void allowIOResumption() {
+        if (appender instanceof DataFileAppender) {
+            DataFileAppender dataFileAppender = (DataFileAppender)appender;
+            dataFileAppender.shutdown = false;
+        }
+    }
+
     public enum PreallocationStrategy {
         SPARSE_FILE,
         OS_KERNEL_COPY,
@@ -213,6 +229,8 @@ public class Journal {
     protected PreallocationScope preallocationScope = PreallocationScope.ENTIRE_JOURNAL;
     protected PreallocationStrategy preallocationStrategy = PreallocationStrategy.SPARSE_FILE;
     private File osKernelCopyTemplateFile = null;
+    private ByteBuffer preAllocateDirectBuffer = null;
+
     protected JournalDiskSyncStrategy journalDiskSyncStrategy = JournalDiskSyncStrategy.ALWAYS;
 
     public interface DataFileRemovedListener {
@@ -274,13 +292,24 @@ public class Journal {
             }
         }
 
-        if (preallocationScope != PreallocationScope.NONE && preallocationStrategy == PreallocationStrategy.OS_KERNEL_COPY) {
-            // create a template file that will be used to pre-allocate the journal files
-            if (osKernelCopyTemplateFile == null) {
-                osKernelCopyTemplateFile = createJournalTemplateFile();
+        if (preallocationScope != PreallocationScope.NONE) {
+            switch (preallocationStrategy) {
+                case SPARSE_FILE:
+                    break;
+                case OS_KERNEL_COPY: {
+                    osKernelCopyTemplateFile = createJournalTemplateFile();
+                }
+                break;
+                case CHUNKED_ZEROS: {
+                    preAllocateDirectBuffer = allocateDirectBuffer(PREALLOC_CHUNK_SIZE);
+                }
+                break;
+                case ZEROS: {
+                    preAllocateDirectBuffer = allocateDirectBuffer(getMaxFileLength());
+                }
+                break;
             }
         }
-
         scheduler = Executors.newScheduledThreadPool(1, new ThreadFactory() {
             @Override
             public Thread newThread(Runnable r) {
@@ -306,8 +335,9 @@ public class Journal {
         }
 
         // ensure we don't report unused space of last journal file in size metric
-        if (totalLength.get() > maxFileLength && lastAppendLocation.get().getOffset() > 0) {
-            totalLength.addAndGet(lastAppendLocation.get().getOffset() - maxFileLength);
+        int lastFileLength = dataFiles.getTail().getLength();
+        if (totalLength.get() > lastFileLength && lastAppendLocation.get().getOffset() > 0) {
+            totalLength.addAndGet(lastAppendLocation.get().getOffset() - lastFileLength);
         }
 
         cleanupTask = scheduler.scheduleAtFixedRate(new Runnable() {
@@ -321,18 +351,29 @@ public class Journal {
         LOG.trace("Startup took: "+(end-start)+" ms");
     }
 
+    private ByteBuffer allocateDirectBuffer(int size) {
+        ByteBuffer buffer = ByteBuffer.allocateDirect(size);
+        buffer.put(EOF_RECORD);
+        return buffer;
+    }
+
     public void preallocateEntireJournalDataFile(RecoverableRandomAccessFile file) {
 
         if (PreallocationScope.NONE != preallocationScope) {
 
-            if (PreallocationStrategy.OS_KERNEL_COPY == preallocationStrategy) {
-                doPreallocationKernelCopy(file);
-            } else if (PreallocationStrategy.ZEROS == preallocationStrategy) {
-                doPreallocationZeros(file);
-            } else if (PreallocationStrategy.CHUNKED_ZEROS == preallocationStrategy) {
-                doPreallocationChunkedZeros(file);
-            } else {
-                doPreallocationSparseFile(file);
+            try {
+                if (PreallocationStrategy.OS_KERNEL_COPY == preallocationStrategy) {
+                    doPreallocationKernelCopy(file);
+                } else if (PreallocationStrategy.ZEROS == preallocationStrategy) {
+                    doPreallocationZeros(file);
+                } else if (PreallocationStrategy.CHUNKED_ZEROS == preallocationStrategy) {
+                    doPreallocationChunkedZeros(file);
+                } else {
+                    doPreallocationSparseFile(file);
+                }
+            } catch (Throwable continueWithNoPrealloc) {
+                // error on preallocation is non fatal, and we don't want to leak the journal handle
+                LOG.error("cound not preallocate journal data file", continueWithNoPrealloc);
             }
         }
     }
@@ -356,12 +397,10 @@ public class Journal {
     }
 
     private void doPreallocationZeros(RecoverableRandomAccessFile file) {
-        ByteBuffer buffer = ByteBuffer.allocate(maxFileLength);
-        buffer.put(EOF_RECORD);
-        buffer.rewind();
+        preAllocateDirectBuffer.rewind();
         try {
             FileChannel channel = file.getChannel();
-            channel.write(buffer);
+            channel.write(preAllocateDirectBuffer);
             channel.force(false);
             channel.position(0);
         } catch (ClosedByInterruptException ignored) {
@@ -372,10 +411,8 @@ public class Journal {
     }
 
     private void doPreallocationKernelCopy(RecoverableRandomAccessFile file) {
-        try {
-            RandomAccessFile templateRaf = new RandomAccessFile(osKernelCopyTemplateFile, "rw");
+        try (RandomAccessFile templateRaf = new RandomAccessFile(osKernelCopyTemplateFile, "rw");){
             templateRaf.getChannel().transferTo(0, getMaxFileLength(), file.getChannel());
-            templateRaf.close();
         } catch (ClosedByInterruptException ignored) {
             LOG.trace("Could not preallocate journal file with kernel copy", ignored);
         } catch (FileNotFoundException e) {
@@ -401,22 +438,19 @@ public class Journal {
     }
 
     private void doPreallocationChunkedZeros(RecoverableRandomAccessFile file) {
-
-        ByteBuffer buffer = ByteBuffer.allocate(PREALLOC_CHUNK_SIZE);
-        buffer.put(EOF_RECORD);
-        buffer.rewind();
-
+        preAllocateDirectBuffer.limit(preAllocateDirectBuffer.capacity());
+        preAllocateDirectBuffer.rewind();
         try {
             FileChannel channel = file.getChannel();
 
             int remLen = maxFileLength;
             while (remLen > 0) {
-                if (remLen < buffer.remaining()) {
-                    buffer.limit(remLen);
+                if (remLen < preAllocateDirectBuffer.remaining()) {
+                    preAllocateDirectBuffer.limit(remLen);
                 }
-                int writeLen = channel.write(buffer);
+                int writeLen = channel.write(preAllocateDirectBuffer);
                 remLen -= writeLen;
-                buffer.rewind();
+                preAllocateDirectBuffer.rewind();
             }
 
             channel.force(false);
@@ -437,21 +471,19 @@ public class Journal {
     }
 
     public boolean isUnusedPreallocated(DataFile dataFile) throws IOException {
-        int firstBatchRecordSize = -1;
         if (preallocationScope == PreallocationScope.ENTIRE_JOURNAL_ASYNC) {
-            Location location = new Location();
-            location.setDataFileId(dataFile.getDataFileId());
-            location.setOffset(0);
-
             DataFileAccessor reader = accessorPool.openDataFileAccessor(dataFile);
             try {
-                firstBatchRecordSize = checkBatchRecord(reader, location.getOffset());
+                byte[] firstFewBytes = new byte[BATCH_CONTROL_RECORD_HEADER.length];
+                reader.readFully(0, firstFewBytes);
+                ByteSequence bs = new ByteSequence(firstFewBytes);
+                return bs.startsWith(EOF_RECORD);
             } catch (Exception ignored) {
             } finally {
                 accessorPool.closeDataFileAccessor(reader);
             }
         }
-        return firstBatchRecordSize == 0;
+        return false;
     }
 
     protected Location recoveryCheck(DataFile dataFile) throws IOException {
@@ -461,9 +493,15 @@ public class Journal {
 
         DataFileAccessor reader = accessorPool.openDataFileAccessor(dataFile);
         try {
+            RandomAccessFile randomAccessFile = reader.getRaf().getRaf();
+            randomAccessFile.seek(0);
+            final long totalFileLength = randomAccessFile.length();
+            byte[] data = new byte[getWriteBatchSize()];
+            ByteSequence bs = new ByteSequence(data, 0, randomAccessFile.read(data));
+
             while (true) {
-                int size = checkBatchRecord(reader, location.getOffset());
-                if (size >= 0 && location.getOffset() + BATCH_CONTROL_RECORD_SIZE + size <= dataFile.getLength()) {
+                int size = checkBatchRecord(bs, randomAccessFile);
+                if (size >= 0 && location.getOffset() + BATCH_CONTROL_RECORD_SIZE + size <= totalFileLength) {
                     if (size == 0) {
                         // eof batch record
                         break;
@@ -474,10 +512,10 @@ public class Journal {
                     // Perhaps it's just some corruption... scan through the
                     // file to find the next valid batch record. We
                     // may have subsequent valid batch records.
-                    int nextOffset = findNextBatchRecord(reader, location.getOffset() + 1);
-                    if (nextOffset >= 0) {
+                    if (findNextBatchRecord(bs, randomAccessFile) >= 0) {
+                        int nextOffset = Math.toIntExact(randomAccessFile.getFilePointer() - bs.remaining());
                         Sequence sequence = new Sequence(location.getOffset(), nextOffset - 1);
-                        LOG.warn("Corrupt journal records found in '" + dataFile.getFile() + "' between offsets: " + sequence);
+                        LOG.warn("Corrupt journal records found in '{}' between offsets: {}", dataFile.getFile(), sequence);
                         dataFile.corruptedBlocks.add(sequence);
                         location.setOffset(nextOffset);
                     } else {
@@ -507,41 +545,34 @@ public class Journal {
         return location;
     }
 
-    private int findNextBatchRecord(DataFileAccessor reader, int offset) throws IOException {
-        ByteSequence header = new ByteSequence(BATCH_CONTROL_RECORD_HEADER);
-        byte data[] = new byte[1024*4];
-        ByteSequence bs = new ByteSequence(data, 0, reader.read(offset, data));
-
+    private int findNextBatchRecord(ByteSequence bs, RandomAccessFile reader) throws IOException {
+        final ByteSequence header = new ByteSequence(BATCH_CONTROL_RECORD_HEADER);
         int pos = 0;
         while (true) {
-            pos = bs.indexOf(header, pos);
+            pos = bs.indexOf(header, 0);
             if (pos >= 0) {
-                return offset + pos;
+                bs.setOffset(bs.offset + pos);
+                return pos;
             } else {
                 // need to load the next data chunck in..
-                if (bs.length != data.length) {
+                if (bs.length != bs.data.length) {
                     // If we had a short read then we were at EOF
                     return -1;
                 }
-                offset += bs.length - BATCH_CONTROL_RECORD_HEADER.length;
-                bs = new ByteSequence(data, 0, reader.read(offset, data));
-                pos = 0;
+                bs.setOffset(bs.length - BATCH_CONTROL_RECORD_HEADER.length);
+                bs.reset();
+                bs.setLength(bs.length + reader.read(bs.data, bs.length, bs.data.length - BATCH_CONTROL_RECORD_HEADER.length));
             }
         }
     }
 
-    public int checkBatchRecord(DataFileAccessor reader, int offset) throws IOException {
-        byte controlRecord[] = new byte[BATCH_CONTROL_RECORD_SIZE];
-
-        try (DataByteArrayInputStream controlIs = new DataByteArrayInputStream(controlRecord);) {
-
-            reader.readFully(offset, controlRecord);
-
-            // check for journal eof
-            if (Arrays.equals(EOF_RECORD, Arrays.copyOfRange(controlRecord, 0, EOF_RECORD.length))) {
-                // eof batch
-                return 0;
-            }
+    private int checkBatchRecord(ByteSequence bs, RandomAccessFile reader) throws IOException {
+        ensureAvailable(bs, reader, EOF_RECORD.length);
+        if (bs.startsWith(EOF_RECORD)) {
+            return 0; // eof
+        }
+        ensureAvailable(bs, reader, BATCH_CONTROL_RECORD_SIZE);
+        try (DataByteArrayInputStream controlIs = new DataByteArrayInputStream(bs)) {
 
             // Assert that it's a batch record.
             for (int i = 0; i < BATCH_CONTROL_RECORD_HEADER.length; i++) {
@@ -552,32 +583,60 @@ public class Journal {
 
             int size = controlIs.readInt();
             if (size < 0 || size > Integer.MAX_VALUE - (BATCH_CONTROL_RECORD_SIZE + EOF_RECORD.length)) {
-                return -1;
+                return -2;
             }
 
-            if (isChecksum()) {
+            long expectedChecksum = controlIs.readLong();
+            Checksum checksum = null;
+            if (isChecksum() && expectedChecksum > 0) {
+                checksum = new Adler32();
+            }
 
-                long expectedChecksum = controlIs.readLong();
-                if (expectedChecksum == 0) {
-                    // Checksuming was not enabled when the record was stored.
-                    // we can't validate the record :(
-                    return size;
-                }
+            // revert to bs to consume data
+            bs.setOffset(controlIs.position());
+            int toRead = size;
+            while (toRead > 0) {
+                if (bs.remaining() >= toRead) {
+                    if (checksum != null) {
+                        checksum.update(bs.getData(), bs.getOffset(), toRead);
+                    }
+                    bs.setOffset(bs.offset + toRead);
+                    toRead = 0;
+                } else {
+                    if (bs.length != bs.data.length) {
+                        // buffer exhausted
+                        return  -3;
+                    }
 
-                byte data[] = new byte[size];
-                reader.readFully(offset + BATCH_CONTROL_RECORD_SIZE, data);
-
-                Checksum checksum = new Adler32();
-                checksum.update(data, 0, data.length);
-
-                if (expectedChecksum != checksum.getValue()) {
-                    return -1;
+                    toRead -= bs.remaining();
+                    if (checksum != null) {
+                        checksum.update(bs.getData(), bs.getOffset(), bs.remaining());
+                    }
+                    bs.setLength(reader.read(bs.data));
+                    bs.setOffset(0);
                 }
             }
+            if (checksum != null && expectedChecksum != checksum.getValue()) {
+                return -4;
+            }
+
             return size;
         }
     }
 
+    private void ensureAvailable(ByteSequence bs, RandomAccessFile reader, int required) throws IOException {
+        if (bs.remaining() < required) {
+            bs.reset();
+            int read = reader.read(bs.data, bs.length, bs.data.length - bs.length);
+            if (read < 0) {
+                if (bs.remaining() == 0) {
+                    throw new EOFException("request for " + required + " bytes reached EOF");
+                }
+            }
+            bs.setLength(bs.length + read);
+        }
+    }
+
     void addToTotalLength(int size) {
         totalLength.addAndGet(size);
     }
@@ -803,6 +862,10 @@ public class Journal {
     }
 
     public Location getNextLocation(Location location) throws IOException, IllegalStateException {
+        return getNextLocation(location, null);
+    }
+
+    public Location getNextLocation(Location location, Location limit) throws IOException, IllegalStateException {
         Location cur = null;
         while (true) {
             if (cur == null) {
@@ -842,6 +905,10 @@ public class Journal {
                 } else {
                     cur.setDataFileId(dataFile.getDataFileId().intValue());
                     cur.setOffset(0);
+                    if (limit != null && cur.compareTo(limit) >= 0) {
+                        LOG.trace("reached limit: {} at: {}", limit, cur);
+                        return null;
+                    }
                 }
             }
 
@@ -849,6 +916,9 @@ public class Journal {
             DataFileAccessor reader = accessorPool.openDataFileAccessor(dataFile);
             try {
                 reader.readLocationDetails(cur);
+            } catch (EOFException eof) {
+                LOG.trace("EOF on next: " + location + ", cur: " + cur);
+                throw eof;
             } finally {
                 accessorPool.closeDataFileAccessor(reader);
             }
