@@ -785,6 +785,9 @@ class DBManager(val parent:LevelDBStore) {
     if( info.getDestination!=null ) {
       record.setDestinationName(info.getDestination.getQualifiedName)
     }
+    if ( info.getSubscribedDestination!=null) {
+      record.setSubscribedDestinationName(info.getSubscribedDestination.getQualifiedName)
+    }
     val collection = new CollectionRecord.Bean()
     collection.setType(SUBSCRIPTION_COLLECTION_TYPE)
     collection.setKey(lastCollectionKey.incrementAndGet())
@@ -854,8 +857,11 @@ class DBManager(val parent:LevelDBStore) {
           if( sr.hasSelector ) {
             info.setSelector(sr.getSelector)
           }
-          if(sr.hasDestinationName) {
-            info.setSubscribedDestination(ActiveMQDestination.createDestination(sr.getDestinationName, ActiveMQDestination.TOPIC_TYPE))
+          if( sr.hasDestinationName ) {
+            info.setDestination(ActiveMQDestination.createDestination(sr.getDestinationName, ActiveMQDestination.TOPIC_TYPE))
+          }
+          if( sr.hasSubscribedDestinationName ) {
+            info.setSubscribedDestination(ActiveMQDestination.createDestination(sr.getSubscribedDestinationName, ActiveMQDestination.TOPIC_TYPE))
           }
 
           var sub = DurableSubscription(key, sr.getTopicKey, info)
