@@ -186,10 +186,10 @@ public class TopicSubscriptionZeroPrefetchTest {
         doTestTopicConsumerPrefetchZeroConcurrentProduceConsume(ackMode, false);
     }
 
-    private void doTestTopicConsumerPrefetchZeroConcurrentProduceConsume(final int ackMode, final boolean commitBatch) throws Exception {
+    private void doTestTopicConsumerPrefetchZeroConcurrentProduceConsume(int ackMode, boolean commitBatch) throws Exception {
 
         ActiveMQTopic consumerDestination = new ActiveMQTopic(getTopicName() + "?consumer.retroactive=true&consumer.prefetchSize=0");
-        final Session consumerSession = connection.createSession(ackMode == Session.SESSION_TRANSACTED, ackMode);
+        Session consumerSession = connection.createSession(ackMode == Session.SESSION_TRANSACTED, ackMode);
         consumer = consumerSession.createConsumer(consumerDestination);
 
         final int MSG_COUNT = 2000;
